>From b955ff8624a298e374fc713fb359b6d70cdcc48e Mon Sep 17 00:00:00 2001
From: Michael Friedrich <michael.friedrich@univie.ac.at>
Date: Sat, 30 Apr 2011 13:35:59 +0200
Subject: [PATCH 1/4] fix race condition on flexible downtime commands when duration not set or zero

it may happen that an external command sets a
flexible downtime with duration not set or zero.

a flexible downtime is triggered between start and
end time and needs a duration to be set and non zero.

without the fix, the scheduled downtime starts and then
tries to calculate the end time out of the set duration
for the downtime and scheduling a new event.
the time passed over there is in the past, and causes the
core to get into a loop.

to prevent this misbehavior, a duration of null or zero is
not allowed and triggered as error in base/commands.c not
setting any flexible downtime.
---
 Changelog       |    1 +
 base/commands.c |   11 +++++++++++
 2 files changed, 12 insertions(+), 0 deletions(-)

diff --git a/Changelog b/Changelog
index c767649..299f306 100644
--- a/Changelog
+++ b/Changelog
@@ -22,6 +22,7 @@ FIXES
 * Fixed bug with passive host checks being incorrectly sent to event brokers as active checks
 * Fixed bug where passive host check status updates were not being propagated to event brokers
 * Reverted 'Fix for retaining host display name and alias, as well as service display name' as configuration information stored incorrectly over a reload
+* fix race condition on flexible downtime commands when duration not set or zero (Michael Friedrich)
 
 WARNINGS
 
diff --git a/base/commands.c b/base/commands.c
index 2b5dfa6..c201772 100644
--- a/base/commands.c
+++ b/base/commands.c
@@ -2505,6 +2505,17 @@ int cmd_schedule_downtime(int cmd, time_t entry_time, char *args){
 	if((comment_data=my_strtok(NULL,";"))==NULL)
 		return ERROR;
 
+	/* check if flexible downtime demanded and duration set
+	   to non-zero.
+	   according to the documentation, a flexible downtime is
+	   started between start and end time and will last for
+	   "duration" seconds. strtoul converts a NULL value to 0
+	   so if set to 0, bail out as a duration>0 is needed. 	   */
+
+	if(fixed==0 && duration==0)
+		return ERROR;
+
+
 	/* duration should be auto-calculated, not user-specified */
 	if(fixed>0)
 		duration=(unsigned long)(end_time-start_time);
-- 
1.7.1

